/* Software virtual memory system */

#ifndef _Ivfile
#define _Ivfile 1

#include "misc.h"

/* Minimum page size for MS-DOS is 128 (for 32K vheaders table) or 256 (for
 * 64K vheaders table) */

#define PGSIZE 8192           /* Page size in bytes (Must be power of 2) */
#define LPGSIZE 13            /* LOG base 2 of PGSIZE */
#define HTSIZE 128            /* Entries in hash table (Must be power of 2) */

typedef struct vpage VPAGE;
typedef struct vfile VFILE;

/* Page header */

struct vpage
{
	VPAGE *next;              /* Next page with same hash value */
	VFILE *vfile;             /* Owner vfile */
	int addr;                 /* Address of this page */
	int count;                /* Reference count */
	int dirty;                /* Set if page changed */
	char *data;               /* The data in the page */
};

/* File structure */

struct vfile
{
	LINK(VFILE) link;         /* Doubly linked list of vfiles */
	int size;                 /* Number of bytes in physical file */
	int alloc;                /* Number of bytes allocated to file */
	int writeable;            /* Set if we can write */
	int addr;                 /* File address of page */
	char *bufp;               /* Buffer pointer */
	char *vpage;              /* Buffer pointer points in here */
	int left;                 /* Space left in bufp */
	int lv;                   /* Amount of append space at end of buffer */
};

extern char *vbase;           /* Data first entry in vheader refers to */
extern VPAGE **vheaders;      /* Array of headers */

/* VFILE *vtmp(V);
 *
 * Open a temporary virtual file.  File goes away when closed.  No actual
 * file is generated if everything fits in memory.
 */
VFILE *vtmp();

/* int vsize(VFILE *);
 *
 * Return size of file
 */
#define vsize(vfile) ((vfile)->left<(vfile)->lv ? (vfile)->alloc+(vfile)->lv-(vfile)->left : (vfile)->alloc)

/* void vclose(VFILE *vfile);
 *
 * Close a file.
 */
void vclose();

/* char *vlock(VFILE *vfile,int addr);
 *
 * Translate virtual address to physical address.  'addr' does not have
 * to be on any particular alignment, but if you wish to access more than
 * a single byte, you have to be aware of where page boundaries are (virtual
 * address multiples of PGSIZE).
 *
 * The page containing the data is locked in memory (so that it won't be
 * freed or used for something else) until 'vunlock' is used.
 *
 * Warning:  If you allocate more than one page and use (change) them out of
 * order, vflsh will screw up if writing past the end of a file is illegal
 * in the host filesystem.
 *
 * Also:  This function does not allocate space to the file.  Use vallocm()
 * for that.  You can vlock() pages past the allocated size of the file, but
 * be careful when you do this (you normally shouldn't- the only time you
 * ever might want to is to implement your own version of vallocm()).
 */
char *vlock();

/* VPAGE *vheader(char *);
 * Return address of page header for given page
 */
#define vheader(p) (vheaders[((unsigned)(p)-(unsigned)(vbase))>>LPGSIZE])

/* void vchanged(char *);
 *
 * Indicate that a vpage was changed so that it will be written back to the
 * file.  Any physical address which falls within the page may be given.
 */
#define vchanged(vpage) (vheader(vpage)->dirty=1)

/* void vunlock(char *);
 * Unreference a vpage (call one vunlock for every vlock)
 * Any physical address which falls within the page may be given.
 */
#define vunlock(vpage)  (--vheader(vpage)->count)

/* void vupcount(char *);
 * Indicate that another reference is being made to a vpage
 */
#define vupcount(vpage) (++vheader(vpage)->count)

/* int vallocm(VFILE *vfile,int size);
 *
 * Allocate space at end of file
 *
 * Returns file address of beginning of allocated space
 */
int vallocm();

#endif
